@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -63,7 +63,7 @@ def force_enabled(codec_name):
     return envbool("XPRA_SOUND_CODEC_ENABLE_%s" % codec_name.upper().replace("+", "_"), False)
 
 
-NAME_TO_SRC_PLUGIN = {
+NAME_TO_SRC_PLUGIN: dict[str, str] = {
     "auto"          : "autoaudiosrc",
     "alsa"          : "alsasrc",
     "oss"           : "osssrc",
@@ -77,12 +77,12 @@ NAME_TO_SRC_PLUGIN = {
 }
 SRC_TO_NAME_PLUGIN = reverse_dict(NAME_TO_SRC_PLUGIN)
 SRC_HAS_DEVICE_NAME = ["alsasrc", "osssrc", "oss4src", "jackaudiosrc", "pulsesrc", "directsoundsrc", "osxaudiosrc"]
-PLUGIN_TO_DESCRIPTION = {
+PLUGIN_TO_DESCRIPTION: dict[str, str] = {
     "pulsesrc"      : "Pulseaudio",
     "jacksrc"       : "JACK Audio Connection Kit",
 }
 
-NAME_TO_INFO_PLUGIN = {
+NAME_TO_INFO_PLUGIN: dict[str, str] = {
     "auto"          : "Automatic audio source selection",
     "alsa"          : "Advanced Linux Sound Architecture",
     "oss"           : "OSS audio cards",
@@ -430,7 +430,7 @@ def get_source_plugins() -> list[str]:
 
 
 def get_default_source() -> str:
-    source = os.environ.get("XPRA_SOUND_SRC")
+    source = os.environ.get("XPRA_SOUND_SRC", "")
     sources = get_source_plugins()
     if source:
         if source not in sources:
@@ -475,7 +475,7 @@ def get_sink_plugins() -> list[str]:
 
 
 def get_default_sink_plugin() -> str:
-    sink = os.environ.get("XPRA_SOUND_SINK")
+    sink = os.environ.get("XPRA_SOUND_SINK", "")
     sinks = get_sink_plugins()
     if sink:
         if sink not in sinks:
@@ -510,7 +510,7 @@ def get_test_defaults(*_args) -> dict[str, Any]:
 WARNED_MULTIPLE_DEVICES = False
 
 
-def get_pulse_defaults(device_name_match=None, want_monitor_device=True,
+def get_pulse_defaults(device_name_match="", want_monitor_device=True,
                        input_or_output=None, remote=None, env_device_name=None) -> dict[str, Any]:
     try:
         device = get_pulse_device(device_name_match, want_monitor_device, input_or_output, remote, env_device_name)
@@ -536,10 +536,64 @@ def get_pulse_defaults(device_name_match=None, want_monitor_device=True,
     return {"device": device}
 
 
-def get_pulse_device(device_name_match=None, want_monitor_device=True,
-                     input_or_output=None, remote=None, env_device_name=None) -> str:
+# noinspection PySimplifyBooleanCheck
+def filter_ignored(devices: dict, input_or_output: bool | None) -> dict:
+    if input_or_output is True:
+        ignore = IGNORED_INPUT_DEVICES
+    elif input_or_output is False:
+        ignore = IGNORED_OUTPUT_DEVICES
+    else:
+        ignore = IGNORED_INPUT_DEVICES+IGNORED_OUTPUT_DEVICES
+    if ignore and devices:
+        # filter out the ignore list:
+        filtered = {}
+        for k, v in devices.items():
+            kl = bytestostr(k).strip().lower()
+            vl = bytestostr(v).strip().lower()
+            if kl not in ignore and vl not in ignore:
+                filtered[k] = v
+        return filtered
+    return devices
+
+
+def filter_name(devices: dict, device_type_str: str, device_name_match: str, env_device: str) -> dict:
+    filters = []
+    matches = {}
+    for match in (device_name_match, env_device):
+        if not match:
+            continue
+        if match != env_device:
+            filters.append(match)
+        match = match.lower()
+        log("trying to match '%s' in devices=%s", match, devices)
+        matches = {
+            k: v for k, v in devices.items()
+            if (bytestostr(k).strip().lower().find(match) >= 0 or bytestostr(v).strip().lower().find(match) >= 0)
+        }
+        # log("matches(%s, %s)=%s", devices, match, matches)
+        if len(matches) == 1:
+            log("found name match for '%s': %s", match, tuple(matches.items())[0])
+            break
+        elif len(matches) > 1:
+            log.warn("Warning: Pulseaudio %s device name filter '%s'", device_type_str, match)
+            log.warn(" matched %i devices", len(matches))
+    if filters or matches:
+        if not matches:
+            log.warn("Warning: Pulseaudio %s device name filters:", device_type_str)
+            log.warn(" %s", csv("'%s'" % x for x in filters))
+            log.warn(" did not match any of the devices found:")
+            for k, v in devices.items():
+                log.warn(" * '%s'", k)
+                log.warn("   '%s'", v)
+            return {}
+        return matches
+    return devices
+
+
+def get_pulse_device(device_name_match="", want_monitor_device=True,
+                     input_or_output=None, remote=None, env_device_name="") -> str:
     """
-        choose the device to use
+    choose the device to use
     """
     log("get_pulse_device%s", (device_name_match, want_monitor_device, input_or_output, remote, env_device_name))
     try:
@@ -579,62 +633,16 @@ def get_pulse_device(device_name_match=None, want_monitor_device=True,
         device_type_str += " monitor"
     devices = get_pa_device_options(want_monitor_device, input_or_output)
     log("found %i pulseaudio %s devices: %s", len(devices), device_type_str, devices)
-    if input_or_output is True:
-        ignore = IGNORED_INPUT_DEVICES
-    elif input_or_output is False:
-        ignore = IGNORED_OUTPUT_DEVICES
-    else:
-        ignore = IGNORED_INPUT_DEVICES+IGNORED_OUTPUT_DEVICES
-    if ignore and devices:
-        # filter out the ignore list:
-        filtered = {}
-        for k, v in devices.items():
-            kl = bytestostr(k).strip().lower()
-            vl = bytestostr(v).strip().lower()
-            if kl not in ignore and vl not in ignore:
-                filtered[k] = v
-        devices = filtered
-
+    devices = filter_ignored(devices, input_or_output)
     if not devices:
         log.error("Error: audio forwarding is disabled")
         log.error(" could not detect any Pulseaudio %s devices", device_type_str)
         return ""
 
-    env_device = None
-    if env_device_name:
-        env_device = os.environ.get(env_device_name)
+    env_device = os.environ.get(env_device_name, "") if env_device_name else ""
     # try to match one of the devices using the device name filters:
     if len(devices) > 1:
-        filters = []
-        matches = {}
-        for match in (device_name_match, env_device):
-            if not match:
-                continue
-            if match != env_device:
-                filters.append(match)
-            match = match.lower()
-            log("trying to match '%s' in devices=%s", match, devices)
-            matches = {
-                k: v for k, v in devices.items()
-                if (bytestostr(k).strip().lower().find(match) >= 0 or bytestostr(v).strip().lower().find(match) >= 0)
-            }
-            # log("matches(%s, %s)=%s", devices, match, matches)
-            if len(matches) == 1:
-                log("found name match for '%s': %s", match, tuple(matches.items())[0])
-                break
-            elif len(matches) > 1:
-                log.warn("Warning: Pulseaudio %s device name filter '%s'", device_type_str, match)
-                log.warn(" matched %i devices", len(matches))
-        if filters or matches:
-            if not matches:
-                log.warn("Warning: Pulseaudio %s device name filters:", device_type_str)
-                log.warn(" %s", csv("'%s'" % x for x in filters))
-                log.warn(" did not match any of the devices found:")
-                for k, v in devices.items():
-                    log.warn(" * '%s'", k)
-                    log.warn("   '%s'", v)
-                return ""
-            devices = matches
+        devices = filter_name(devices, device_type_str, device_name_match, env_device)
 
     # still have too many devices to choose from?
     if len(devices) > 1:
@@ -650,15 +658,8 @@ def get_pulse_device(device_name_match=None, want_monitor_device=True,
         global WARNED_MULTIPLE_DEVICES
         if not WARNED_MULTIPLE_DEVICES:
             WARNED_MULTIPLE_DEVICES = True
-            dtype = "audio"
-            if want_monitor_device:
-                dtype = "output monitor"
-            elif input_or_output is False:
-                dtype = "audio output"
-            elif input_or_output is True:
-                dtype = "audio input"
-            log.info("found %i %s devices:", len(devices), dtype)
-            for k,v in devices.items():
+            log.info("found %i audio %s devices:", len(devices), device_type_str)
+            for k, v in devices.items():
                 log.info(" * %s", bytestostr(v))
                 log.info("   %s", bytestostr(k))
             if not env_device:  # used already!
@@ -675,7 +676,7 @@ def get_pulse_device(device_name_match=None, want_monitor_device=True,
     return device
 
 
-def get_pulse_source_defaults(device_name_match=None, want_monitor_device=True, remote=None) -> dict[str, Any]:
+def get_pulse_source_defaults(device_name_match="", want_monitor_device=True, remote=None) -> dict[str, Any]:
     return get_pulse_defaults(device_name_match, want_monitor_device,
                               input_or_output=not want_monitor_device, remote=remote,
                               env_device_name=XPRA_PULSE_SOURCE_DEVICE_NAME)
@@ -820,7 +821,7 @@ def loop_warning_messages(mode="speaker") -> list[str]:
     ]
 
 
-def main():
+def main() -> None:
     from xpra.platform import program_context
     from xpra.log import enable_color
     with program_context("GStreamer-Info", "GStreamer Information"):
