@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 from time import monotonic
 from collections import deque
+from collections.abc import Sequence
 from threading import Lock
 from typing import Any, Literal, NoReturn
 
@@ -23,9 +24,10 @@ from xpra.audio.gstreamer_util import (
     MP3, CODEC_ORDER, QUEUE_LEAK,
     GST_QUEUE_NO_LEAK, MS_TO_NS, DEFAULT_SINK_PLUGIN_OPTIONS,
 )
-from xpra.gtk.gobject import one_arg_signal
+from xpra.util.gobject import one_arg_signal
 from xpra.net.compression import decompress_by_name
 from xpra.scripts.config import InitExit
+from xpra.common import SizedBuffer
 from xpra.os_util import gi_import
 from xpra.util.str_fn import csv
 from xpra.util.env import envint, envbool
@@ -47,7 +49,7 @@ NON_AUTO_SINK_ATTRIBUTES: dict[str, Any] = {
     "qos": True,
 }
 
-SINK_DEFAULT_ATTRIBUTES = {
+SINK_DEFAULT_ATTRIBUTES: dict[str, dict[str, str]] = {
     "pulsesink": {"client-name": "Xpra"},
 }
 
@@ -63,7 +65,7 @@ UNDERRUN_MIN_LEVEL = max(0, envint("XPRA_SOUND_UNDERRUN_MIN_LEVEL", 150))
 CLOCK_SYNC = envbool("XPRA_CLOCK_SYNC", False)
 
 
-def uncompress_data(data, metadata: dict):
+def uncompress_data(data: bytes, metadata: dict) -> SizedBuffer:
     if not data or not metadata:
         return data
     compress = metadata.get("compress")
@@ -82,7 +84,7 @@ class AudioSink(AudioPipeline):
         "eos": one_arg_signal,
     }
 
-    def __init__(self, sink_type=None, sink_options=None, codecs=(), codec_options=None, volume=1.0):
+    def __init__(self, sink_type: str, sink_options: dict, codecs: Sequence[str], codec_options: dict, volume=1.0):
         if not sink_type:
             sink_type = get_default_sink_plugin()
         if sink_type not in get_sink_plugins():
@@ -401,7 +403,7 @@ class AudioSink(AudioPipeline):
             return
         data = uncompress_data(data, metadata)
         for x in packet_metadata:
-            self.do_add_data(x)
+            self.do_add_data(x, {})
         if self.do_add_data(data, metadata):
             self.rec_queue_level(data)
             self.set_max_level()
@@ -415,7 +417,7 @@ class AudioSink(AudioPipeline):
                     self.refill = False
         self.emit_info()
 
-    def do_add_data(self, data, metadata=None) -> bool:
+    def do_add_data(self, data, metadata: dict) -> bool:
         # having a timestamp causes problems with the queue and overruns:
         log("do_add_data(%s bytes, %s) queue_state=%s", len(data), metadata, self.queue_state)
         self.save_to_file(data)
@@ -508,10 +510,11 @@ def main() -> int:
         from xpra.audio import gstreamer_util
         gstreamer_util.QUEUE_LEAK = GST_QUEUE_NO_LEAK
         gstreamer_util.QUEUE_SILENT = True
-        ss = AudioSink(codecs=codecs)
 
-        def eos(*args):
-            print("eos%s" % (args,))
+        ss = AudioSink("", sink_options={}, codecs=codecs, codec_options={})
+
+        def eos(*eos_args) -> None:
+            print("eos%s" % (eos_args,))
             GLib.idle_add(glib_mainloop.quit)
 
         ss.connect("eos", eos)
@@ -534,7 +537,7 @@ def main() -> int:
         signal.signal(signal.SIGINT, deadly_signal)
         signal.signal(signal.SIGTERM, deadly_signal)
 
-        def check_for_end(*_args):
+        def check_for_end(*_args) -> bool:
             qtime = ss.queue.get_property("current-level-time") // MS_TO_NS
             if qtime <= 0:
                 log.info("underrun (end of stream)")
