@@ -1,6 +1,6 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
-# Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,16 +11,16 @@ from typing import Any
 from collections.abc import Sequence
 
 from xpra.util.objects import typedict
-from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, sorted_nicely, bytestostr, hexstr
+from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, ellipsize, sorted_nicely, bytestostr, hexstr
 from xpra.util.env import envint, first_time
-from xpra.common import ConnectionMessage, disconnect_is_an_error, noop
+from xpra.common import ConnectionMessage, disconnect_is_an_error, BACKWARDS_COMPATIBLE
 from xpra.os_util import gi_import, get_hex_uuid, POSIX, OSX
-from xpra.util.io import stderr_print
-from xpra.net.common import PacketType
+from xpra.util.io import stderr_print, load_binary_file
+from xpra.net.common import Packet, PacketElement
 from xpra.util.stats import std_unit
 from xpra.client.base.client import EXTRA_TIMEOUT
 from xpra.client.base.gobject import GObjectXpraClient
-from xpra.exit_codes import ExitCode, ExitValue
+from xpra.exit_codes import ExitCode, ExitValue, exit_str
 from xpra.log import Logger
 
 log = Logger("gobject", "client")
@@ -28,7 +28,7 @@ log = Logger("gobject", "client")
 GLib = gi_import("GLib")
 
 
-def errwrite(msg) -> None:
+def errwrite(msg: str) -> None:
     stderr_print(msg)
 
 
@@ -47,15 +47,18 @@ class CommandConnectClient(GObjectXpraClient):
         self.file_transfer = False
         self.printing = False
         self.command_timeout = None
+        self.last_server_event: Sequence[PacketElement] = ()
         # don't bother with many of these things for one-off commands:
-        for x in ("ui_client", "windows", "webcam", "keyboard", "mouse", "network-state"):
+        SKIP = ["ui_client", "windows", "webcam", "keyboard", "pointer", "network-state"]
+        if BACKWARDS_COMPATIBLE:
+            SKIP.append("mouse")
+        for x in SKIP:
             self.hello_extra[x] = False
         # for newer versions, it is easier:
         self.hello_extra["wants"] = []
 
-    def setup_connection(self, conn):
-        self._packet_handlers["setting-change"] = noop
-        protocol = super().setup_connection(conn)
+    def make_protocol(self, conn):
+        protocol = super().make_protocol(conn)
         if conn.timeout > 0:
             self.command_timeout = GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
         return protocol
@@ -69,7 +72,7 @@ class CommandConnectClient(GObjectXpraClient):
             self.command_timeout = None
             GLib.source_remove(ct)
 
-    def _process_connection_lost(self, packet: PacketType) -> None:
+    def _process_connection_lost(self, packet: Packet) -> None:
         log("_process_connection_lost%s", packet)
         # override so we don't log a warning
         # "command clients" are meant to exit quickly by losing the connection
@@ -91,6 +94,7 @@ class CommandConnectClient(GObjectXpraClient):
         # * we don't care about sending anything back after hello
         log("server_capabilities: %s", Ellipsizer(caps))
         log("protocol state: %s", self._protocol.save_state())
+        self.init_authenticated_packet_handlers()
         self.cancel_command_timeout()
         self.do_command(caps)
         return True
@@ -107,7 +111,6 @@ class SendCommandConnectClient(CommandConnectClient):
     """
 
     def server_connection_established(self, caps) -> bool:
-        assert self.parse_encryption_capabilities(caps), "encryption failure"
         assert self.parse_network_capabilities(caps), "network capabilities failure"
         return super().server_connection_established(caps)
 
@@ -125,19 +128,20 @@ class HelloRequestClient(SendCommandConnectClient):
         return super().make_hello_base() | self.hello_request()
 
     def timeout(self, *_args) -> None:
+        log("%s client reached the timeout", type(self))  # pragma: no cover
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: server did not disconnect us")
 
-    def hello_request(self):  # pragma: no cover
+    def hello_request(self) -> dict[str, Any]:  # pragma: no cover
         raise NotImplementedError(f"`hello_request` not implemented in {self!r}")
 
     def do_command(self, caps: typedict) -> None:
         self.quit(ExitCode.OK)
 
-    def _process_disconnect(self, packet: PacketType) -> None:
+    def _process_disconnect(self, packet: Packet) -> None:
         # overridden method, so we can avoid printing a warning,
         # we haven't received the hello back from the server
         # but that's fine for a request client
-        info = tuple(repr(x) for x in packet[1:])
+        info = tuple(str(x) for x in packet[1:])
         reason = info[0]
         if disconnect_is_an_error(reason):
             self.server_disconnect_warning(*info)
@@ -168,8 +172,11 @@ class ScreenshotXpraClient(HelloRequestClient):
     def timeout(self, *_args) -> None:
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: did not receive the screenshot")
 
-    def _process_screenshot(self, packet: PacketType) -> None:
-        (w, h, encoding, _, img_data) = packet[1:6]
+    def _process_screenshot(self, packet: Packet) -> None:
+        w = packet.get_u16(1)
+        h = packet.get_u16(2)
+        encoding = packet.get_str(3)
+        img_data = packet.get_buffer(5)
         assert encoding == "png", "expected png screenshot data but got %s" % encoding
         if not img_data:
             self.warn_and_quit(ExitCode.OK,
@@ -187,7 +194,7 @@ class ScreenshotXpraClient(HelloRequestClient):
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._ui_packet_handlers["screenshot"] = self._process_screenshot
+        self.add_packets("screenshot")
 
 
 class InfoXpraClient(CommandConnectClient):
@@ -195,9 +202,12 @@ class InfoXpraClient(CommandConnectClient):
         it queries the server with an 'info' request
     """
 
-    def __init__(self, opts):
+    def __init__(self, opts, subsystems: Sequence[str] = ()):
         super().__init__(opts)
         self.hello_extra["request"] = "info"
+        if subsystems:
+            self.hello_extra["subsystems"] = subsystems
+        log("info request for subsystems %s", subsystems or "'all'")
 
     def timeout(self, *_args):
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: did not receive the info")
@@ -207,10 +217,12 @@ class InfoXpraClient(CommandConnectClient):
             self.quit(ExitCode.NO_DATA)
             return
 
+        LINE_LIMIT = envint("XPRA_INFO_LINE_LIMIT", 320)
+
         def print_fn(s) -> None:
             sys.stdout.write(f"{s}\n")
 
-        def prettify(k, v):
+        def prettify(k, v) -> str:
             ks = str(k)
             if ks.endswith("xid"):
                 try:
@@ -223,14 +235,21 @@ class InfoXpraClient(CommandConnectClient):
                 if ks.endswith(".data"):
                     return hexstr(v)
                 return bytestostr(v)
+            elif isinstance(v, (int, bool, float)):
+                return str(v)
             elif isinstance(v, (tuple, list)):
                 eltypes = set(type(x) for x in v)
-                if len(eltypes) == 1 and tuple(eltypes)[0] in (int, bool, float):
-                    return str(v)
-                return type(v)(prettify(k, x) for x in v)
+                if len(eltypes) == 1:
+                    eltype = tuple(eltypes)[0]
+                    if eltype in (int, bool, float):
+                        return repr_ellipsized(v, LINE_LIMIT)
+                    if eltype is str:
+                        csv_str = csv(repr(x) for x in v)
+                        return ellipsize(csv_str, LINE_LIMIT)
+                return repr_ellipsized(type(v)(prettify(k, x) for x in v), LINE_LIMIT)
             elif isinstance(v, str) and v.isprintable():
                 return v
-            return repr(v)
+            return repr_ellipsized(v, LINE_LIMIT)
 
         def print_dict(d: dict, path=""):
             for k in sorted_nicely(d.keys()):
@@ -287,7 +306,7 @@ class ConnectTestXpraClient(CommandConnectClient):
     def timeout(self, *_args) -> None:
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: no server response")
 
-    def _process_connection_lost(self, _packet: PacketType) -> None:
+    def _process_connection_lost(self, _packet: Packet) -> None:
         # we should always receive a hello back and call do_command,
         # which sets the correct exit code, landing here is an error:
         self.quit(ExitCode.FAILURE)
@@ -323,16 +342,15 @@ class MonitorXpraClient(SendCommandConnectClient):
     def do_command(self, caps: typedict) -> None:
         log.info("waiting for server events")
 
-    def _process_server_event(self, packet: PacketType) -> None:
+    def _process_server_event(self, packet: Packet) -> None:
         log.info(": ".join(str(x) for x in packet[1:]))
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._packet_handlers["server-event"] = self._process_server_event
-        self._packet_handlers["ping"] = self._process_ping
+        self.add_packets("server-event", "ping")
 
-    def _process_ping(self, packet: PacketType) -> None:
-        echotime = packet[1]
+    def _process_ping(self, packet: Packet) -> None:
+        echotime = packet.get_u64(1)
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
 
@@ -351,7 +369,7 @@ class InfoTimerClient(MonitorXpraClient):
         self.info_timer = 0
 
     def run(self) -> ExitValue:
-        from xpra.gtk.signals import register_os_signals
+        from xpra.util.glib import register_os_signals
         register_os_signals(self.signal_handler, "")
         v = super().run()
         self.log("run()=%s" % v)
@@ -391,18 +409,18 @@ class InfoTimerClient(MonitorXpraClient):
 
     def init_packet_handlers(self) -> None:
         MonitorXpraClient.init_packet_handlers(self)
-        self.add_packet_handler("info-response", self._process_info_response, False)
+        self.add_packets("info-response")
 
-    def _process_server_event(self, packet: PacketType) -> None:
+    def _process_server_event(self, packet: Packet) -> None:
         self.log("server event: %s" % (packet,))
         self.last_server_event = packet[1:]
         self.update_screen()
 
-    def _process_info_response(self, packet: PacketType) -> None:
+    def _process_info_response(self, packet: Packet) -> None:
         self.log("info response: %s" % repr_ellipsized(packet))
         self.cancel_info_timer()
         self.info_request_pending = False
-        self.server_last_info = typedict(packet[1])
+        self.server_last_info = typedict(packet.get_dict(1))
         self.server_last_info_time = monotonic()
         # log.info("server_last_info=%s", self.server_last_info)
         self.update_screen()
@@ -418,7 +436,7 @@ class InfoTimerClient(MonitorXpraClient):
         self.update_screen()
         return True
 
-    def update_screen(self):
+    def update_screen(self) -> None:
         raise NotImplementedError()
 
 
@@ -491,16 +509,16 @@ class ShellXpraClient(SendCommandConnectClient):
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._packet_handlers["shell-reply"] = self._process_shell_reply
-        self._packet_handlers["ping"] = self._process_ping
+        self.add_packets("shell-reply", "ping")
 
-    def _process_ping(self, packet: PacketType) -> None:
-        echotime = packet[1]
+    def _process_ping(self, packet: Packet) -> None:
+        echotime = packet.get_u64(1)
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
-    def _process_shell_reply(self, packet: PacketType) -> None:
-        fd = int(packet[1])
-        message = packet[2]
+    @staticmethod
+    def _process_shell_reply(packet: Packet) -> None:
+        fd = packet.get_u8(1)
+        message = packet.get_str(2)
         if fd == 1:
             stream = sys.stdout
         elif fd == 2:
@@ -510,7 +528,7 @@ class ShellXpraClient(SendCommandConnectClient):
         s = str(message)
         if s.endswith("\n"):
             s = s[:-1]
-        stream.write("%s" % s)
+        stream.write(s)
         stream.flush()
         if fd == 2:
             stream.write("\n")
@@ -523,10 +541,12 @@ class VersionXpraClient(HelloRequestClient):
     """
 
     def hello_request(self) -> dict[str, Any]:
-        return {
+        req: dict[str, Any] = {
             "request": "version",
-            "full-version-request": True,
         }
+        if BACKWARDS_COMPATIBLE:
+            req["full-version-request"] = True
+        return req
 
     def parse_network_capabilities(self, *_args) -> bool:
         # don't bother checking anything - this could generate warnings
@@ -545,8 +565,8 @@ class VersionXpraClient(HelloRequestClient):
 class ControlXpraClient(CommandConnectClient):
     """ Allows us to send commands to a server.
     """
-
-    def set_command_args(self, command: Sequence[str]) -> None:
+    def __init__(self, opts, command: Sequence[str]):
+        super().__init__(opts)
         self.command = command
 
     def timeout(self, *_args) -> None:
@@ -554,7 +574,7 @@ class ControlXpraClient(CommandConnectClient):
 
     def do_command(self, caps: typedict) -> None:
         cr = caps.tupleget("command_response")
-        if cr is None:
+        if not cr or len(cr) < 2:
             self.warn_and_quit(ExitCode.UNSUPPORTED, "server does not support control command")
             return
         code = int(cr[0])
@@ -573,13 +593,49 @@ class ControlXpraClient(CommandConnectClient):
         return capabilities
 
 
+class RunClient(CommandConnectClient):
+    """
+    Requests the server to start a command
+    """
+
+    def __init__(self, opts, command: Sequence[str]):
+        super().__init__(opts)
+        self.command = command
+
+    def timeout(self, *_args) -> None:
+        self.warn_and_quit(ExitCode.TIMEOUT, "timeout: server did not respond")
+
+    def do_command(self, caps: typedict) -> None:
+        cr = typedict(caps.dictget("run_response") or {})
+        if not cr:
+            self.warn_and_quit(ExitCode.UNSUPPORTED, "server does not support run command")
+            return
+        code = cr.intget("code")
+        pid = cr.intget("pid")
+        message = cr.strget("message")
+        if code != 0:
+            log.warn("server returned error code %s: %s", code, exit_str(code))
+            self.warn_and_quit(ExitCode.REMOTE_ERROR, " %s" % message)
+            return
+        sys.stdout.write(f"command started with pid {pid}\n")
+        self.quit(ExitCode.OK)
+
+    def make_hello(self) -> dict[str, Any]:
+        capabilities = super().make_hello()
+        log("make_hello() adding run request '%s' to %s", self.command, capabilities)
+        capabilities["run"] = tuple(self.command)
+        capabilities["request"] = "run"
+        return capabilities
+
+
 class PrintClient(SendCommandConnectClient):
     """
     Allows us to send a file to the server for printing.
     """
 
-    def set_command_args(self, command: Sequence[str]) -> None:
-        log("set_command_args(%s)", command)
+    def __init__(self, opts, command: Sequence[str]):
+        super().__init__(opts)
+        self.client_type = "Python/GObject/Print"
         self.filename = command[0]
         # print command arguments:
         # filename, file_data, mimetype, source_uuid, title, printer, no_copies, print_options_str = packet[1:9]
@@ -595,7 +651,6 @@ class PrintClient(SendCommandConnectClient):
         else:
             if not self.validate_file_size(os.path.getsize(self.filename)):
                 return
-            from xpra.util.io import load_binary_file
             self.file_data = load_binary_file(self.filename)
             log("read %i bytes from %s", len(self.file_data), self.filename)
         if not self.validate_file_size(len(self.file_data)):
@@ -610,9 +665,6 @@ class PrintClient(SendCommandConnectClient):
                                std_unit(size), std_unit(self.file_size_limit)))
         return False
 
-    def client_type(self) -> str:
-        return "Python/GObject/Print"
-
     def timeout(self, *_args) -> None:
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: server did not respond")
 
@@ -694,7 +746,7 @@ class DetachXpraClient(HelloRequestClient):
 class WaitForDisconnectXpraClient(DetachXpraClient):
     """ we just want the connection to close """
 
-    def _process_disconnect(self, _packet: PacketType) -> None:
+    def _process_disconnect(self, _packet: Packet) -> None:
         self.quit(ExitCode.OK)
 
 
@@ -708,7 +760,7 @@ class RequestStartClient(HelloRequestClient):
         errwrite(".")
         return not self.connection_established
 
-    def _process_connection_lost(self, packet: PacketType) -> None:
+    def _process_connection_lost(self, packet: Packet) -> None:
         errwrite("\n")
         super()._process_connection_lost(packet)
 
