@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2018-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,16 +9,17 @@ import os.path
 from subprocess import Popen, PIPE
 from collections.abc import Callable
 
-from xpra.gtk.signals import register_os_signals
+from xpra.util.glib import register_os_signals
 from xpra.gtk.window import add_close_accel
 from xpra.gtk.widget import imagebutton, label, setfont
 from xpra.gtk.pixbuf import get_icon_pixbuf
 from xpra.util.str_fn import repr_ellipsized
 from xpra.common import noop
-from xpra.os_util import POSIX, OSX, WIN32, gi_import
-from xpra.util.system import is_Wayland, platform_name
+from xpra.os_util import OSX, WIN32, gi_import
+from xpra.util.system import platform_name, is_X11
 from xpra.util.stats import std_unit_dec
-from xpra.scripts.config import get_defaults, str_to_bool, OPTION_TYPES, FALSE_OPTIONS, TRUE_OPTIONS
+from xpra.scripts.config import get_defaults, OPTION_TYPES
+from xpra.util.parsing import TRUE_OPTIONS, FALSE_OPTIONS, str_to_bool
 from xpra.client.gtk3.menu_helper import BANDWIDTH_MENU_OPTIONS
 from xpra.util.objects import AtomicInteger
 from xpra.util.thread import start_thread
@@ -69,11 +70,11 @@ def l(text):  # noqa: E743
 
 
 def link_btn(link: str, text="", icon_name="question.png"):
-    def open_link():
+    def open_link() -> None:
         import webbrowser
         webbrowser.open(link)
 
-    def help_clicked(*args):
+    def help_clicked(*args) -> None:
         log("help_clicked%s opening '%s'", args, link)
         start_thread(open_link, "open-link", True)
 
@@ -252,7 +253,7 @@ class StartSession(Gtk.Window):
         hbox = Gtk.HBox(homogeneous=False, spacing=20)
         vbox.pack_start(hbox, False, True, 20)
 
-        def btn(text, tooltip, callback, default=False):
+        def btn(text, tooltip, callback, default=False) -> Gtk.Button:
             ib = imagebutton(text, tooltip=tooltip, clicked_callback=callback, icon_size=32,
                              default=default, label_font="sans 16")
             hbox.pack_start(ib)
@@ -433,7 +434,9 @@ class StartSession(Gtk.Window):
                 self.category_box.hide()
                 self.entry_label.set_text("Command:" if seamless else "Desktop Environment:")
                 self.entry_box.show_all()
-                self.exit_with_children_cb.set_sensitive(bool(self.entry.get_text()))
+                has_entry = bool(self.entry.get_text())
+                self.exit_with_children_cb.set_active(has_entry)
+                self.exit_with_children_cb.set_sensitive(has_entry)
 
     def populate_category(self) -> None:
         self.categories = {}
@@ -597,10 +600,10 @@ class StartSession(Gtk.Window):
         shadow = self.shadow_btn.get_active()
         seamless = self.seamless_btn.get_active()
         if seamless:
-            return "start"
+            return "seamless"
         if shadow:
             return "shadow"
-        return "start-desktop"
+        return "desktop"
 
     def get_run_command(self, attach=False) -> list[str]:
         localhost = self.localhost_btn.get_active()
@@ -929,7 +932,7 @@ class FeaturesWindow(SessionOptions):
             "force": ("force",),
         })
         self.sep()
-        # "https://github.com/Xpra-org/xpra/blob/master/docs/Features/Notifications.md")
+        # "https://github.com/Xpra-org/xpra/blob/master/docs/Features/Notification.md")
         self.bool_cb("Xpra's System Tray", "tray")
         self.bool_cb("Forward System Trays", "system-tray")
         self.bool_cb("Notifications", "notifications")
@@ -953,7 +956,7 @@ class FeaturesWindow(SessionOptions):
             "to-client": "to client only",
             "disabled": "disabled",
         })
-        if POSIX and not OSX and not is_Wayland():
+        if is_X11():
             self.bool_cb("XSettings", "xsettings")
         self.vbox.show_all()
 
@@ -1008,9 +1011,9 @@ class DisplayWindow(SessionOptions):
             pixel_depths[8] = "8"
         for pd in (16, 24, 30, 32):
             pixel_depths[pd] = str(pd)
-        if self.run_mode == "start-desktop":
+        if self.run_mode == "desktop":
             size_options: dict[str, str] = {
-                "yes": "auto",
+                "auto": "auto",
             }
             for size in SCREEN_SIZES:
                 try:
@@ -1071,7 +1074,7 @@ class EncodingWindow(SessionOptions):
             0.15: "normal",
             0.5: "slow",
         })
-        from xpra.client.mixins.encodings import get_core_encodings
+        from xpra.client.subsystem.encoding import get_core_encodings
         encodings = ["auto", "rgb"] + list(get_core_encodings())
         encodings.remove("rgb24")
         encodings.remove("rgb32")
@@ -1190,7 +1193,7 @@ def main(options=None) -> int:  # pragma: no cover
     from xpra.gtk.util import init_display_source
     with program_context("xpra-start-gui", "Xpra Start GUI"):
         enable_color()
-        init_display_source()
+        init_display_source(False)
         init()
         gui = StartSession(options)
         register_os_signals(gui.app_signal)
