@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (C) 2020-2023 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,13 +11,13 @@ Gtk = gi_import("Gtk")
 GLib = gi_import("GLib")
 
 
-def main():
+def main() -> int:
     with program_context("file-chooser", "File Chooser"):
         file_filter = Gtk.FileFilter()
         file_filter.set_name("Xpra")
         file_filter.add_pattern("*.xpra")
         window = None
-        from xpra.gtk.signals import quit_on_signals
+        from xpra.gtk.util import quit_on_signals
         quit_on_signals("file chooser test window")
         choose_file(window, "test", Gtk.FileChooserAction.OPEN, Gtk.STOCK_OPEN)
         return 0
