@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2015-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2015 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -197,6 +197,19 @@ def get_rectangle_info(rect: Gdk.Rectangle) -> dict[str, int]:
     return info
 
 
+def get_average_monitor_refresh_rate() -> int:
+    rates = {}
+    display = Gdk.Display.get_default()
+    for m in range(display.get_n_monitors()):
+        monitor = display.get_monitor(m)
+        log(f"monitor {m} ({monitor.get_model()}) refresh-rate={monitor.get_refresh_rate()}")
+        rates[m] = monitor.get_refresh_rate()
+    rate = -1
+    if rates:
+        rate = round(min(rates.values()) / 1000)
+    return rate
+
+
 def get_monitor_info(monitor: Gdk.Monitor) -> dict[str, Any]:
     geom = monitor.get_geometry()
     info: dict[str, Any] = get_rectangle_info(geom)
@@ -234,15 +247,15 @@ def get_monitors_info(xscale: float = 1.0, yscale: float = 1.0) -> dict[int, Any
                     continue
                 if isinstance(value, Gdk.Rectangle):
                     value = (
-                        round(xscale * value.x),
-                        round(yscale * value.y),
-                        round(xscale * value.width),
-                        round(yscale * value.height),
+                        round(value.x / xscale),
+                        round(value.y / yscale),
+                        round(value.width / xscale),
+                        round(value.height / yscale),
                     )
                 elif attr == "width-mm":
-                    value = round(xscale * value)
+                    value = round(value / xscale)
                 elif attr == "height-mm":
-                    value = round(yscale * value)
+                    value = round(value / yscale)
                 elif attr == "subpixel-layout":
                     value = {
                         Gdk.SubpixelLayout.UNKNOWN: "unknown",
@@ -316,7 +329,7 @@ def get_screens_info() -> dict[int, dict]:
     return info
 
 
-def get_screen_sizes(xscale: float = 1, yscale: float = 1):
+def get_screen_sizes(xscale: float = 1, yscale: float = 1) -> list[tuple[int, int]]:
     from xpra.platform.gui import get_workarea, get_workareas
 
     def xs(v) -> int:
@@ -412,7 +425,7 @@ def get_screen_sizes(xscale: float = 1, yscale: float = 1):
     ydpi = dpi(sh, hmm)
     if xdpi < MIN_DPI or xdpi > MAX_DPI or ydpi < MIN_DPI or ydpi > MAX_DPI:
         screenlog(f"ignoring invalid DPI {xdpi},{ydpi} from screen size {wmm}x{hmm}mm")
-        if os.environ.get("WAYLAND_DISPLAY"):
+        if os.environ.get("WAYLAND_DISPLAY", ""):
             screenlog(" (wayland display?)")
         if n_monitors > 0:
             wmm = 0
