@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -203,7 +203,7 @@ WIN32_KEYBOARDS: dict[int, tuple[str, str]] = {
     0x00000c0c: ("ca", "Canadian French (Legacy)"),
     0x00011009: ("ca", "Canadian Multilingual Standard"),
     0x0000085f: ("fr", "Central Atlas Tamazight"),
-    0x00000429: ("ku", "Central Kurdish"),
+    0x00000492: ("ku", "Central Kurdish"),
     0x0000045c: ("us", "Cherokee Nation"),
     0x0001045c: ("us", "Cherokee Nation Phonetic"),
     0x00000804: ("cn", "Chinese (Simplified) - US Keyboard"),
@@ -298,9 +298,8 @@ WIN32_KEYBOARDS: dict[int, tuple[str, str]] = {
     0x000f0c00: ("??", "Old Italic"),
     0x000e0c00: ("??", "Osmanya"),
     0x00000463: ("af", "Pashto (Afghanistan)"),
-    # duplicate of 'ku'
-    # 0x00000429: ("ir", "Persian"),
-    0xa0000429: ("ir", "Persian (Standard)"),
+    0x00000429: ("ir", "Persian"),
+    0x00050429: ("ir", "Persian (Standard)"),
     0x000a0c00: ("cn", "Phags-pa"),
     0x00010415: ("pl", "Polish (214)"),
     0x00000415: ("pl", "Polish (Programmers)"),
@@ -393,6 +392,14 @@ for win32layout in WIN32_LAYOUTS.values():
             layout_variants.append(variant)
 
 
+# make it easier to resolve a 2-letter code into a meaningful description:
+LAYOUT_NAMES: dict[str, str] = {}
+for code3, country, name, codepage, code2, variants in WIN32_LAYOUTS.values():
+    if code2 in LAYOUT_NAMES:
+        continue
+    LAYOUT_NAMES[code2] = name
+
+
 def parse_xkbmap_query(xkbmap_query: str) -> dict[str, str]:
     """ parses the output of "setxkbmap -query" into a dict """
     import re
