@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2012 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -310,12 +310,12 @@ def check_lists(props: dict[str, Any], force_enable=False) -> bool:
     return bool(whitelisted) or not bool(blocklisted)
 
 
-def check_PyOpenGL_support(force_enable) -> dict[str, Any]:
+def check_PyOpenGL_support(force_enable: bool) -> dict[str, Any]:
     redirected_loggers: dict[str, tuple[Logger, list, bool]] = {}
     try:
         if CRASH:
-            import ctypes
-            ctypes.string_at(0)
+            from xpra.os_util import crash
+            crash()
             raise RuntimeError("should have crashed!")
         if TIMEOUT > 0:
             import time
@@ -389,11 +389,12 @@ def check_PyOpenGL_support(force_enable) -> dict[str, Any]:
             else:
                 log.info(msg)
         if missing_accelerators:
-            if len(missing_accelerators) == 1 and missing_accelerators[0] == "numpy_formathandler":
+            missing_str = csv(missing_accelerators)
+            if missing_str == "numpy_formathandler":
                 log_fn = log.debug
             else:
                 log_fn = log.info
-            log_fn("OpenGL accelerate missing: %s", csv(missing_accelerators))
+            log_fn(f"OpenGL accelerate missing: {missing_str}")
 
         for msg in recs("plugins"):
             log(f"plugins msg={msg}")
@@ -504,7 +505,10 @@ def do_check_PyOpenGL_support(force_enable) -> dict[str, Any]:
     if safe and match_list(props, GREYLIST, "greylist"):
         props["enable"] = False
         props["message"] = "driver found in greylist"
-    props.update(get_context_info())
+    try:
+        props.update(get_context_info())
+    except Exception as e:
+        unsafe(f"error querying context flags: {e}")
     return props
 
 
