@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -19,6 +19,67 @@ from xpra.util.str_fn import bytestostr
 log = Logger("keyboard", "posix")
 
 
+def get_locale_status() -> dict[str, str]:
+    # parse the output into a dictionary:
+    # $ localectl status
+    # System Locale: LANG=en_GB.UTF-8
+    # VC Keymap: gb
+    # X11 Layout: gb
+    from subprocess import getoutput  # pylint: disable=import-outside-toplevel
+    out = getoutput("localectl status")
+    if not out:
+        return {}
+    locale = {}
+    for line in bytestostr(out).splitlines():
+        parts = line.lstrip(" ").split(": ")
+        if len(parts) == 2:
+            locale[parts[0]] = parts[1]
+    log("locale(%s)=%s", out, locale)
+    return locale
+
+
+def get_all_x11_layouts() -> dict[str, str]:
+    repository = "/usr/share/X11/xkb/rules/base.xml"
+    x11_layouts: dict[str, str] = {}
+    if os.path.exists(repository):
+        try:
+            import lxml.etree  # pylint: disable=import-outside-toplevel
+        except ImportError:
+            log("cannot parse xml", exc_info=True)
+        else:
+            try:
+                with open(repository, encoding="latin1") as f:
+                    tree = lxml.etree.parse(f)  # pylint: disable=c-extension-no-member @UndefinedVariable
+            except Exception:
+                log.error(f"Error parsing {repository}", exc_info=True)
+            else:
+                for layout in list(tree.xpath("//layout")):
+                    layout_str = str(layout.xpath("./configItem/name")[0].text)
+                    x11_layouts[layout] = layout_str
+                    # for variant in layout.xpath("./variantlist/variant/configItem/name"):
+                    #    variant_name = variant.text
+                return x11_layouts
+    from subprocess import Popen, PIPE  # pylint: disable=import-outside-toplevel
+    try:
+        proc = Popen(["localectl", "list-x11-keymap-layouts"], stdout=PIPE, stderr=PIPE)
+        out = proc.communicate()[0]
+        log("get_all_x11_layouts() proc=%s", proc)
+        log("get_all_x11_layouts() returncode=%s", proc.returncode)
+        if proc.wait() == 0:
+            for line in out.splitlines():
+                layout = line.decode().split("/")[-1]
+                if layout:
+                    x11_layouts[layout] = layout
+            return x11_layouts
+    except OSError:
+        log("get_all_x11_layouts()", exc_info=True)
+    return {"us": "English"}
+
+
+def get_keyboard_device():
+    return None
+
+
 class Keyboard(KeyboardBase):
 
     def __init__(self):
@@ -38,7 +99,7 @@ class Keyboard(KeyboardBase):
 
     def init_vars(self) -> None:
         super().init_vars()
-        self.keymap_modifiers = None
+        self.keymap_modifiers = ()
         self.keyboard_bindings = None
         self.__input_sources: dict[str, int] = {}
         try:
@@ -136,7 +197,7 @@ class Keyboard(KeyboardBase):
         return "posix.Keyboard"
 
     def get_keymap_modifiers(self) -> tuple[dict, list[str], list[str]]:
-        if self.keymap_modifiers is None:
+        if not self.keymap_modifiers:
             self.keymap_modifiers = self.do_get_keymap_modifiers()
         return self.keymap_modifiers
 
@@ -147,7 +208,7 @@ class Keyboard(KeyboardBase):
                     log.warn("Warning: incomplete keymap support under Wayland")
                 return {}, [], ["mod2", ]
             return {}, [], []
-        from xpra.gtk.error import xlog
+        from xpra.x11.error import xlog
         with xlog:
             mod_mappings = self.keyboard_bindings.get_modifier_mappings()
             if mod_mappings:
@@ -169,33 +230,15 @@ class Keyboard(KeyboardBase):
     def get_x11_keymap(self) -> dict[int, list[str]]:
         if not self.keyboard_bindings:
             return {}
-        from xpra.gtk.error import xlog
+        from xpra.x11.error import xlog
         with xlog:
             return self.keyboard_bindings.get_keycode_mappings()
         return {}
 
-    def get_locale_status(self) -> dict[str, str]:
-        # parse the output into a dictionary:
-        # $ localectl status
-        # System Locale: LANG=en_GB.UTF-8
-        # VC Keymap: gb
-        # X11 Layout: gb
-        from subprocess import getoutput  # pylint: disable=import-outside-toplevel
-        out = getoutput("localectl status")
-        if not out:
-            return {}
-        locale = {}
-        for line in bytestostr(out).splitlines():
-            parts = line.lstrip(" ").split(": ")
-            if len(parts) == 2:
-                locale[parts[0]] = parts[1]
-        log("locale(%s)=%s", out, locale)
-        return locale
-
     def get_keymap_spec(self) -> dict[str, str]:
         log("get_keymap_spec() keyboard_bindings=%s", self.keyboard_bindings)
         if is_Wayland() or not self.keyboard_bindings:
-            locale = self.get_locale_status()
+            locale = get_locale_status()
             query_struct = {}
             if locale:
                 layout = locale.get("X11 Layout")
@@ -203,92 +246,43 @@ class Keyboard(KeyboardBase):
                     query_struct["layout"] = layout
             log("query_struct(%s)=%s", locale, query_struct)
             return query_struct
-        from xpra.gtk.error import xsync
+        from xpra.x11.error import xsync
         with xsync:
             query_struct = self.keyboard_bindings.getXkbProperties()
         log("get_keymap_spec()=%r", query_struct)
         return query_struct
 
-    def get_xkb_rules_names_property(self) -> Sequence[str]:
-        # parses the "_XKB_RULES_NAMES" X11 property
-        if not is_X11():
-            return ()
-        xkb_rules_names: list[str] = []
-        # pylint: disable=import-outside-toplevel
-        from xpra.gtk.error import xlog
-        from xpra.x11.common import get_X11_root_property
-        with xlog:
-            prop = get_X11_root_property("_XKB_RULES_NAMES", "STRING")
-            log("get_xkb_rules_names_property() _XKB_RULES_NAMES=%s", prop)
-            # ie: 'evdev\x00pc104\x00gb,us\x00,\x00\x00'
-            if prop:
-                xkb_rules_names = bytestostr(prop).split("\0")
-            # ie: ['evdev', 'pc104', 'gb,us', ',', '', '']
-        log("get_xkb_rules_names_property()=%s", xkb_rules_names)
-        return tuple(xkb_rules_names)
+    def get_all_x11_layouts(self):
+        return get_all_x11_layouts()
 
-    def get_all_x11_layouts(self) -> dict[str, str]:
-        repository = "/usr/share/X11/xkb/rules/base.xml"
-        x11_layouts: dict[str, str] = {}
-        if os.path.exists(repository):
-            try:
-                import lxml.etree  # pylint: disable=import-outside-toplevel
-            except ImportError:
-                log("cannot parse xml", exc_info=True)
-            else:
-                try:
-                    with open(repository, encoding="latin1") as f:
-                        tree = lxml.etree.parse(f)  # pylint: disable=c-extension-no-member @UndefinedVariable
-                except Exception:
-                    log.error(f"Error parsing {repository}", exc_info=True)
-                else:
-                    for layout in list(tree.xpath("//layout")):
-                        layout_str = str(layout.xpath("./configItem/name")[0].text)
-                        x11_layouts[layout] = layout_str
-                        # for variant in layout.xpath("./variantlist/variant/configItem/name"):
-                        #    variant_name = variant.text
-                    return x11_layouts
-        from subprocess import Popen, PIPE  # pylint: disable=import-outside-toplevel
-        try:
-            proc = Popen(["localectl", "list-x11-keymap-layouts"], stdout=PIPE, stderr=PIPE)
-            out = proc.communicate()[0]
-            log("get_all_x11_layouts() proc=%s", proc)
-            log("get_all_x11_layouts() returncode=%s", proc.returncode)
-            if proc.wait() == 0:
-                for line in out.splitlines():
-                    layout = line.decode().split("/")[-1]
-                    if layout:
-                        x11_layouts[layout] = layout
-                return x11_layouts
-        except OSError:
-            log("get_all_x11_layouts()", exc_info=True)
-        return {"us": "English"}
-
-    def get_layout_spec(self):
+    def get_layout_spec(self) -> tuple[str, str, Sequence[str], str, Sequence[str], str]:
+        model = ""
         layout = ""
-        layouts = []
+        layouts = ()
         variant = ""
         options = ""
         if self.keyboard_bindings:
-            from xpra.gtk.error import xsync
+            from xpra.x11.error import xsync
             with xsync:
                 props = self.keyboard_bindings.getXkbProperties()
+            model = props.get("model", "")
             v = props.get("layout")
             variant = props.get("variant", "")
             options = props.get("options", "")
         else:
-            locale = self.get_locale_status()
+            locale = get_locale_status()
             v = locale.get("X11 Layout", "")
         if not v:
             # fallback:
-            props = self.get_xkb_rules_names_property()
+            from xpra.x11.xroot_props import get_xkb_rules_names
+            props = get_xkb_rules_names()
             # ie: ['evdev', 'pc104', 'gb,us', ',', '', '']
             if props and len(props) >= 3:
                 v = props[2]
         if v:
             layouts = v.split(",")
             layout = v
-        return layout, [x for x in layouts], variant, [], options
+        return model, layout, layouts, variant, (), options
 
     def get_keyboard_repeat(self) -> tuple[int, int] | None:
         if self.keyboard_bindings:
@@ -303,13 +297,13 @@ class Keyboard(KeyboardBase):
                 log.estr(e)
         return None
 
-    def update_modifier_map(self, mod_meanings) -> None:
+    def update_modifier_map(self, mod_meanings: dict) -> None:
         try:
             # pylint: disable=import-outside-toplevel
-            from xpra.x11.gtk.keys import grok_modifier_map
+            from xpra.x11.xkbhelper import grok_modifier_map
             self.modifier_map = grok_modifier_map(mod_meanings)
         except ImportError:
             self.modifier_map = MODIFIER_MAP
         # force re-query on next call:
-        self.keymap_modifiers = None
+        self.keymap_modifiers = ()
         log(f"update_modifier_map({mod_meanings}) modifier_map={self.modifier_map}")
