@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,6 +8,7 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
+
 import glob
 import shlex
 import signal
@@ -16,74 +17,17 @@ from typing import NoReturn
 from subprocess import Popen, PIPE, call
 import os.path
 
-from xpra.common import RESOLUTION_ALIASES, DEFAULT_REFRESH_RATE, get_refresh_rate_for_value
-from xpra.scripts.config import InitException, get_Xdummy_confdir, FALSE_OPTIONS
+from xpra.scripts.config import InitException, get_Xdummy_confdir
 from xpra.util.str_fn import csv
-from xpra.util.env import envint, envbool, shellsub, osexpand, get_exec_env
+from xpra.util.env import envint, envbool, shellsub, osexpand, get_exec_env, get_saved_env_var
 from xpra.os_util import getuid, getgid, POSIX, OSX
-from xpra.server.util import setuidgid
-from xpra.util.io import is_writable, pollwait
+from xpra.util.daemon import setuidgid
+from xpra.util.io import is_writable, pollwait, osclose, get_status_output
 from xpra.platform.displayfd import read_displayfd, parse_displayfd
 from xpra.log import Logger
 
-
 VFB_WAIT = envint("XPRA_VFB_WAIT", 3)
 XVFB_EXTRA_ARGS = os.environ.get("XPRA_XVFB_EXTRA_ARGS", "")
-
-
-def parse_resolution(res_str, default_refresh_rate=DEFAULT_REFRESH_RATE//1000) -> tuple[int, int, int] | None:
-    if not res_str:
-        return None
-    s = res_str.upper()       # ie: 4K60
-    res_part = s
-    hz = get_refresh_rate_for_value(str(default_refresh_rate), DEFAULT_REFRESH_RATE)//1000
-    for sep in ("@", "K", "P"):
-        pos = s.find(sep)
-        if 0 < pos < len(s)-1:
-            res_part, hz = s.split(sep, 1)
-            if sep != "@":
-                res_part += sep
-            break
-    w = h = 0
-    if res_part in RESOLUTION_ALIASES:
-        w, h = RESOLUTION_ALIASES[res_part]
-    else:
-        try:
-            parts = tuple(int(x) for x in res_part.replace(",", "x").split("X", 1))
-        except ValueError:
-            raise ValueError(f"failed to parse resolution {res_str!r}") from None
-        if len(parts) != 2:
-            raise ValueError(f"invalid resolution string {res_str!r}")
-        w = parts[0]
-        h = parts[1]
-    return w, h, int(hz)
-
-
-def parse_resolutions(s, default_refresh_rate=DEFAULT_REFRESH_RATE//1000) -> tuple | None:
-    if not s or s.lower() in FALSE_OPTIONS:
-        return None
-    if s.lower() in ("none", "default"):
-        return ()
-    return tuple(parse_resolution(v, default_refresh_rate) for v in s.split(","))
-
-
-def parse_env_resolutions(envkey="XPRA_DEFAULT_VFB_RESOLUTIONS",
-                          single_envkey="XPRA_DEFAULT_VFB_RESOLUTION",
-                          default_res="8192x4096",
-                          default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
-    s = os.environ.get(envkey)
-    if s:
-        return parse_resolutions(s, default_refresh_rate)
-    return (parse_resolution(os.environ.get(single_envkey, default_res), default_refresh_rate), )
-
-
-def get_desktop_vfb_resolutions(default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
-    return parse_env_resolutions("XPRA_DEFAULT_DESKTOP_VFB_RESOLUTIONS",
-                                 "XPRA_DEFAULT_DESKTOP_VFB_RESOLUTION",
-                                 "1280x1024",
-                                 default_refresh_rate=default_refresh_rate)
-
-
 PRIVATE_XAUTH = envbool("XPRA_PRIVATE_XAUTH", False)
 XAUTH_PER_DISPLAY = envbool("XPRA_XAUTH_PER_DISPLAY", True)
 
@@ -98,13 +42,6 @@ def get_vfb_logger() -> Logger:
     return vfb_logger
 
 
-def osclose(fd: int) -> None:
-    try:
-        os.close(fd)
-    except OSError:
-        pass
-
-
 def create_xorg_device_configs(xorg_conf_dir: str, device_uuid: str, uid: int, gid: int) -> None:
     log = get_vfb_logger()
     log("create_xorg_device_configs(%s, %s, %i, %i)", xorg_conf_dir, device_uuid, uid, gid)
@@ -175,7 +112,8 @@ def get_xauthority_path(display_name: str) -> str:
     assert POSIX
     # pylint: disable=import-outside-toplevel
     from xpra.platform.posix.paths import _get_xpra_runtime_dir
-    has_home = os.path.exists(os.path.expanduser("~"))
+    expanded_home = os.path.expanduser("~")
+    has_home = os.path.exists(expanded_home) and is_writable(expanded_home, getuid(), getgid())
     if PRIVATE_XAUTH or (not has_home and os.environ.get("XDG_RUNTIME_DIR")):
         d = _get_xpra_runtime_dir()
         if XAUTH_PER_DISPLAY:
@@ -183,122 +121,222 @@ def get_xauthority_path(display_name: str) -> str:
         else:
             filename = "Xauthority"
     else:
-        if has_home:
-            d = "~/"
+        if has_home and is_writable(expanded_home, getuid(), getgid()):
+            d = expanded_home
         else:
             d = os.environ.get("TMPDIR", "/tmp")
         filename = ".Xauthority"
     return os.path.join(d, filename)
 
 
-def start_Xvfb(xvfb_str: str, vfb_geom, pixel_depth: int, display_name: str, cwd,
-               uid: int, gid: int, username: str, uinput_uuid="") -> tuple[Popen, str]:
-    if not POSIX:
-        raise InitException(f"starting an Xvfb is not supported on {os.name}")
-    if OSX:
-        raise InitException("starting an Xvfb is not supported on MacOS")
-    if not xvfb_str:
-        raise InitException("the 'xvfb' command is not defined")
+def get_xvfb_env(xvfb_executable: str) -> dict[str, str]:
+    keep = tuple(f"^{envname}$" for envname in (
+        "SHELL", "HOSTNAME", "XMODIFIERS",
+        "PWD", "HOME", "USERNAME", "LANG", "TERM", "USER",
+        "XDG_RUNTIME_DIR", "XDG_DATA_DIR",
+        "PATH", "LD_LIBRARY_PATH",
+        "XAUTHORITY",
+        "XPRA_SESSION_DIR",
+    ))
+    env = get_exec_env(keep=keep)
+    if xvfb_executable.endswith("Xephyr"):
+        env["DISPLAY"] = get_saved_env_var("DISPLAY")
+    return env
 
-    log = get_vfb_logger()
-    log("start_Xvfb%s XVFB_EXTRA_ARGS=%s",
-        (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid),
-        XVFB_EXTRA_ARGS)
-    use_display_fd = display_name[0] == 'S'
-    if XVFB_EXTRA_ARGS:
-        xvfb_str += " "+XVFB_EXTRA_ARGS
 
-    subs: dict[str, str] = {}
+def patch_xvfb_command_fps(xvfb_cmd: list[str], fps: int) -> None:
+    # find the ["-fakescreenfps"] argument:
+    try:
+        fps_arg = xvfb_cmd.index("-fakescreenfps")
+        xvfb_cmd[fps_arg + 1] = str(fps)
+    except ValueError:
+        from xpra.util.system import can_use_fakescreenfps
+        if can_use_fakescreenfps():
+            xvfb_cmd += ["-fakescreenfps", str(fps)]
 
-    def pathexpand(s: str) -> str:
-        return osexpand(s, actual_username=username, uid=uid, gid=gid, subs=subs)
-    etc_prefix = os.environ.get("XPRA_INSTALL_PREFIX", "")
-    if etc_prefix.endswith("/usr"):
-        etc_prefix = etc_prefix[:-4]
-    subs |= {
-        "DISPLAY": display_name,
-        "XPRA_LOG_DIR": pathexpand(os.environ.get("XPRA_LOG_DIR")),
-        "XORG_CONFIG_PREFIX": os.environ.get("XORG_CONFIG_PREFIX", etc_prefix),
-    }
 
-    # identify logfile argument if it exists,
-    # as we may have to rename it, or create the directory for it:
-    xvfb_cmd = shlex.split(xvfb_str)
-    if not xvfb_cmd:
-        raise InitException("cannot start Xvfb, the command definition is missing!")
-    # make sure all path values are expanded:
-    xvfb_cmd = [pathexpand(s) for s in xvfb_cmd]
-
-    # try to honour initial geometries if specified:
-    if vfb_geom and xvfb_cmd[0].endswith("Xvfb"):
-        # find the '-screen' arguments:
-        # "-screen 0 8192x4096x24+32"
+def patch_xvfb_command_geometry(xvfb_cmd: list[str], vfb_geom, pixel_depth: int) -> None:
+    if len(vfb_geom) < 2:
+        return
+    xvfb_executable = xvfb_cmd[0]
+    if not (xvfb_executable.endswith("Xvfb") or xvfb_executable.endswith("Xephyr")):
+        return
+    w, h = vfb_geom[:2]
+    pixel_depth = pixel_depth or 32
+    if pixel_depth not in (8, 16, 24, 30, 32):
+        raise ValueError(f"unsupported pixel depth {pixel_depth}")
+    # find the ["-screen"] or ["screen", "0"] arguments:
+    try:
+        screen_arg = xvfb_cmd.index("-screen")
+    except ValueError:
+        # not found!
+        screen_arg = -1
+    if screen_arg > 0:
+        # remove the screen number if specified:
         try:
             no_arg = xvfb_cmd.index("0")
         except ValueError:
             no_arg = -1
-        geom_str = "%sx%s" % ("x".join(str(x) for x in vfb_geom[:2]), pixel_depth)
-        if no_arg > 0 and xvfb_cmd[no_arg-1] == "-screen" and len(xvfb_cmd) > (no_arg+1):
-            # found an existing "-screen" argument for this screen number,
-            # replace it:
-            xvfb_cmd[no_arg+1] = geom_str
-        else:
-            # append:
-            xvfb_cmd += ["-screen", "0", geom_str]
+        if no_arg > 0 and no_arg == screen_arg + 1:
+            xvfb_cmd.pop(no_arg)
+        # remove the "-screen" argument and the geometry that follows:
+        xvfb_cmd.pop(screen_arg)
+        xvfb_cmd.pop(screen_arg)
+    # this is the geometry we want to add:
+    if xvfb_cmd[0].endswith("Xvfb"):
+        # ie: "-screen 0 8192x4096x32"
+        xvfb_cmd += ["-screen", "0", f"{w}x{h}x{pixel_depth}"]
+    else:
+        # play it safe, don't specify the screen number:
+        xvfb_cmd += ["-screen", f"{w}x{h}x{pixel_depth}"]
+
+
+def patch_uinput(xvfb_cmd: list[str]) -> bool:
+    # use uinput:
+    # identify -config xorg.conf argument and replace it with the uinput one:
+    try:
+        config_argindex = xvfb_cmd.index("-config")
+    except ValueError:
+        log = get_vfb_logger()
+        log.warn("Warning: cannot use uinput")
+        log.warn(" '-config' argument not found in the xvfb command")
+        return False
+    if config_argindex + 1 >= len(xvfb_cmd):
+        raise InitException("invalid xvfb command string: -config should not be last")
+    xorg_conf = xvfb_cmd[config_argindex + 1]
+    if xorg_conf.endswith("xorg.conf"):
+        xorg_conf = xorg_conf.replace("xorg.conf", "xorg-uinput.conf")
+        if os.path.exists(xorg_conf):
+            xvfb_cmd[config_argindex + 1] = xorg_conf
+    return True
+
+
+def patch_pixel_depth(xvfb_cmd: list[str], depth: int) -> None:
+    try:
+        config_argindex = xvfb_cmd.index("-depth")
+    except ValueError:
+        # maybe add it?
+        xvfb_executable = xvfb_cmd[0]
+        if xvfb_executable.endswith("Xorg") or xvfb_executable.endswith("Xdummy"):
+            xvfb_cmd.append("-depth")
+            xvfb_cmd.append(str(depth))
+        return
+    if config_argindex + 1 >= len(xvfb_cmd):
+        raise InitException("invalid xvfb command string: -depth should not be last")
+    xvfb_cmd[config_argindex + 1] = str(depth)
+
+
+def get_logfile_arg(xvfb_cmd: list[str]) -> str:
     try:
         logfile_argindex = xvfb_cmd.index('-logfile')
-        if logfile_argindex+1 >= len(xvfb_cmd):
+        if logfile_argindex + 1 >= len(xvfb_cmd):
             raise InitException("invalid xvfb command string: -logfile should not be last")
-        xorg_log_file = xvfb_cmd[logfile_argindex+1]
+        return xvfb_cmd[logfile_argindex+1]
     except ValueError:
-        xorg_log_file = None
-    tmp_xorg_log_file = None
+        return ""
+
+
+def mklogdir(xorg_log_file: str, uid: int, gid: int) -> None:
+    # make sure the Xorg log directory exists:
+    xorg_log_dir = os.path.dirname(xorg_log_file)
+    if os.path.exists(xorg_log_dir):
+        return
+    log = get_vfb_logger()
+    try:
+        log("creating Xorg log dir '%s'", xorg_log_dir)
+        os.mkdir(xorg_log_dir, 0o700)
+        if POSIX and uid != getuid() or gid != getgid():
+            try:
+                os.lchown(xorg_log_dir, uid, gid)
+            except OSError:
+                log("lchown(%s, %i, %i)", xorg_log_dir, uid, gid, exc_info=True)
+    except OSError as e:
+        raise InitException(f"failed to create the Xorg log directory {xorg_log_dir!r}: {e}") from None
+
+
+def start_Xvfb(xvfb_cmd: list[str], vfb_geom, pixel_depth: int, fps: int, display_name: str, cwd,
+               uid: int, gid: int, username: str, uinput_uuid="") -> tuple[Popen, str]:
+    if not POSIX:
+        raise InitException(f"starting an Xvfb is not supported on {os.name}")
+    if not xvfb_cmd:
+        raise InitException("the 'xvfb' command is not defined")
+    if not display_name:
+        raise ValueError("no display name")
+
+    log = get_vfb_logger()
+    log("start_Xvfb%s XVFB_EXTRA_ARGS=%s",
+        (xvfb_cmd, vfb_geom, pixel_depth, fps, display_name, cwd, uid, gid, username, uinput_uuid),
+        XVFB_EXTRA_ARGS)
+    use_display_fd = display_name[0] == "S"
+
+    if OSX and not os.path.exists("/tmp/.X11-unix"):
+        if getuid() == 0:
+            try:
+                os.mkdir("/tmp/.X11-unix", 0o1777)
+            except OSError as e:
+                raise InitException(f"failed to create /tmp/.X11-unix: {e}") from None
+        else:
+            # we can't create it directly, but running XQuartz will create it for us:
+            # best ot use an X11 utility like `xset` rather than `open -a XQuartz`
+            # because XQuartz will also launch an `xterm` - which we do not want.
+            from xpra.platform.paths import get_app_dir
+            base = get_app_dir()
+            xset = os.path.join(base, "Resources", "bin", "xset")
+            if not os.path.exists(xset) or not os.path.isfile(xset):
+                raise InitException(f"xset not found at {xset!r}")
+            log("using xset to start XQuartz and create /tmp/.X11-unix")
+            code, out, err = get_status_output([xset, "-q"])
+            if code != 0:
+                log.warn("Warning: xset failed and returned %i", code)
+                log.warn(" error output: %r", err.strip())
+            log("xset -q output: %r", out.strip())
+            if not os.path.exists("/tmp/.X11-unix"):
+                raise InitException("XQuartz failed to create /tmp/.X11-unix, cannot start Xvfb")
+
+    etc_prefix = os.environ.get("XPRA_INSTALL_PREFIX", "")
+    if etc_prefix.endswith("/usr"):
+        etc_prefix = etc_prefix[:-4]
+
+    subs: dict[str, str] = {
+        "DISPLAY": display_name,
+        "XORG_CONFIG_PREFIX": os.environ.get("XORG_CONFIG_PREFIX", etc_prefix),
+    }
+
+    def pathexpand(s: str) -> str:
+        return osexpand(s, actual_username=username, uid=uid, gid=gid, subs=subs)
+    subs |= {
+        "XPRA_LOG_DIR": pathexpand(os.environ.get("XPRA_LOG_DIR", "")),
+    }
+
+    # identify logfile argument if it exists,
+    # as we may have to rename it, or create the directory for it:
+    # make sure all path values are expanded:
+    xvfb_cmd = [pathexpand(s) for s in xvfb_cmd]
+    if XVFB_EXTRA_ARGS:
+        xvfb_cmd += shlex.split(XVFB_EXTRA_ARGS)
+    if fps > 0:
+        patch_xvfb_command_fps(xvfb_cmd, fps)
+    if pixel_depth > 0:
+        patch_pixel_depth(xvfb_cmd, pixel_depth)
+    if vfb_geom:
+        patch_xvfb_command_geometry(xvfb_cmd, vfb_geom, pixel_depth)
+
+    xorg_log_file = get_logfile_arg(xvfb_cmd)
+    tmp_xorg_log_file = ""
     if xorg_log_file:
         if use_display_fd:
             # keep track of it, so that we can rename it later:
             tmp_xorg_log_file = xorg_log_file
-        # make sure the Xorg log directory exists:
-        xorg_log_dir = os.path.dirname(xorg_log_file)
-        if not os.path.exists(xorg_log_dir):
-            try:
-                log("creating Xorg log dir '%s'", xorg_log_dir)
-                os.mkdir(xorg_log_dir, 0o700)
-                if POSIX and uid != getuid() or gid != getgid():
-                    try:
-                        os.lchown(xorg_log_dir, uid, gid)
-                    except OSError:
-                        log("lchown(%s, %i, %i)", xorg_log_dir, uid, gid, exc_info=True)
-            except OSError as e:
-                raise InitException(f"failed to create the Xorg log directory {xorg_log_dir!r}: {e}") from None
+        mklogdir(xorg_log_file, uid, gid)
 
-    if uinput_uuid:
-        # use uinput:
-        # identify -config xorg.conf argument and replace it with the uinput one:
-        try:
-            config_argindex = xvfb_cmd.index("-config")
-        except ValueError:
-            log.warn("Warning: cannot use uinput")
-            log.warn(" '-config' argument not found in the xvfb command")
-        else:
-            if config_argindex+1 >= len(xvfb_cmd):
-                raise InitException("invalid xvfb command string: -config should not be last")
-            xorg_conf = xvfb_cmd[config_argindex+1]
-            if xorg_conf.endswith("xorg.conf"):
-                xorg_conf = xorg_conf.replace("xorg.conf", "xorg-uinput.conf")
-                if os.path.exists(xorg_conf):
-                    xvfb_cmd[config_argindex+1] = xorg_conf
-            # create uinput device definition files:
-            # (we have to assume that Xorg is configured to use this path..)
-            xorg_conf_dir = pathexpand(get_Xdummy_confdir())
-            create_xorg_device_configs(xorg_conf_dir, uinput_uuid, uid, gid)
+    if uinput_uuid and patch_uinput(xvfb_cmd):
+        # create uinput device definition files:
+        # (we have to assume that Xorg is configured to use this path..)
+        xorg_conf_dir = pathexpand(get_Xdummy_confdir())
+        create_xorg_device_configs(xorg_conf_dir, uinput_uuid, uid, gid)
 
     xvfb_executable = xvfb_cmd[0]
-    if (xvfb_executable.endswith("Xorg") or xvfb_executable.endswith("Xdummy")) and pixel_depth > 0:
-        xvfb_cmd.append("-depth")
-        xvfb_cmd.append(str(pixel_depth))
-    env = get_exec_env(keep=("SHELL", "HOSTNAME", "XMODIFIERS",
-                             "PWD", "HOME", "USERNAME", "LANG", "TERM", "USER",
-                             "XDG_RUNTIME_DIR", "XDG_DATA_DIR", "PATH"))
+    env = get_xvfb_env(xvfb_executable)
     log(f"xvfb env={env}")
     xvfb = None
     try:
@@ -318,26 +356,21 @@ def start_Xvfb(xvfb_str: str, vfb_geom, pixel_depth: int, display_name: str, cwd
                     os.setpgrp()
                     if getuid() == 0 and uid:
                         setuidgid(uid, gid)
-                try:
-                    # pylint: disable=consider-using-with
-                    # pylint: disable=subprocess-popen-preexec-fn
-                    xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
-                                 preexec_fn=preexec, cwd=cwd, env=env, pass_fds=(w_pipe,))
-                except OSError as e:
-                    log("Popen%s", (xvfb_cmd, xvfb_executable, cwd), exc_info=True)
-                    raise InitException(f"failed to execute xvfb command {xvfb_cmd}: {e}") from None
+                # pylint: disable=consider-using-with
+                # pylint: disable=subprocess-popen-preexec-fn
+                xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
+                             preexec_fn=preexec, cwd=cwd, env=env, pass_fds=(w_pipe,))
                 if xvfb.poll() is not None:
                     raise InitException(f"xvfb command has terminated: {xvfb_cmd}")
                 # Read the display number from the pipe we gave to Xvfb
                 try:
-                    buf = read_displayfd(r_pipe)
-                except Exception as e:
+                    buf = read_displayfd(r_pipe, proc=xvfb)
+                except OSError as e:
                     buf = b""
                     log("read_displayfd(%s)", r_pipe, exc_info=True)
                     displayfd_err(f"failed to read displayfd pipe {r_pipe}: {e}")
             finally:
-                osclose(r_pipe)
-                osclose(w_pipe)
+                osclose(r_pipe, w_pipe)
             n = parse_displayfd(buf, displayfd_err)
             if n < 0:
                 displayfd_err(f"failed to parse displayfd output {buf!r}")
@@ -372,6 +405,9 @@ def start_Xvfb(xvfb_str: str, vfb_geom, pixel_depth: int, display_name: str, cwd
             # pylint: disable=subprocess-popen-preexec-fn
             xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
                          stdin=PIPE, preexec_fn=preexec, env=env)
+    except OSError as e:
+        log("Popen%s", (xvfb_cmd, xvfb_executable, cwd), exc_info=True)
+        raise InitException(f"failed to execute xvfb command {xvfb_cmd}: {e}") from None
     except Exception:
         if xvfb and xvfb.poll() is None:
             log.error(" stopping vfb process with pid %i", xvfb.pid)
@@ -382,6 +418,42 @@ def start_Xvfb(xvfb_str: str, vfb_geom, pixel_depth: int, display_name: str, cwd
     return xvfb, display_name
 
 
+def start_xvfb_standalone(xvfb_cmd: list[str], sessions_dir: str, pixel_depth=0, fps=0, display_name="",
+                          daemon=False) -> int:
+    # we may have to tweak the environment to emulate having an xpra session
+    # as the default xvfb commands may refer to $XAUTHORITY and $XPRA_SESSION_DIR
+    xauthority = os.environ.get("XAUTHORITY", "")
+    if "$XAUTHORITY" in xvfb_cmd and not valid_xauth(xauthority):
+        xauthority = osexpand(get_xauthority_path(display_name))
+        os.environ["XAUTHORITY"] = xauthority
+    if "$XPRA_SESSION_DIR" in xvfb_cmd and "XPRA_SESSION_DIR" not in os.environ:
+        from xpra.scripts.session import make_session_dir
+        session_dir = make_session_dir("xvfb", sessions_dir, display_name)
+        os.environ["XPRA_SESSION_DIR"] = session_dir
+    vfb_geom = ()
+    cwd = os.getcwd()
+    try:
+        xvfb, actual_display_name = start_Xvfb(xvfb_cmd, vfb_geom, pixel_depth, fps, display_name,
+                                               cwd, uid=getuid(), gid=getgid(), username="", uinput_uuid="")
+    except Exception:
+        log = get_vfb_logger()
+        log.error("Error starting xvfb", exc_info=True)
+        raise
+    if actual_display_name != display_name:
+        print(f"display {actual_display_name!r} started")
+    print(f"xvfb pid: {xvfb.pid}")
+    if daemon:
+        return 0
+    try:
+        return xvfb.wait()
+    except KeyboardInterrupt:
+        try:
+            xvfb.terminate()
+        except OSError as e:
+            print(f"failed to terminate xvfb process: {e}")
+        return 128-signal.SIGINT
+
+
 def kill_xvfb(xvfb_pid: int) -> None:
     log = get_vfb_logger()
     log.info("killing xvfb with pid %s", xvfb_pid)
@@ -390,7 +462,7 @@ def kill_xvfb(xvfb_pid: int) -> None:
     except OSError as e:
         log.info("failed to kill xvfb process with pid %s:", xvfb_pid)
         log.info(" %s", e)
-    xauthority = os.environ.get("XAUTHORITY")
+    xauthority = os.environ.get("XAUTHORITY", "")
     if PRIVATE_XAUTH and xauthority and os.path.exists(xauthority):
         os.unlink(xauthority)
 
@@ -464,7 +536,7 @@ def xauth_add(filename: str, display_name: str, xauth_data: str, uid: int, gid:
     xauth_args = ["-f", filename, "add", display_name, "MIT-MAGIC-COOKIE-1", xauth_data]
     xauth_cmd = ["xauth"] + xauth_args
     try:
-        def preexec():
+        def preexec() -> None:
             os.setsid()
             if getuid() == 0 and uid:
                 setuidgid(uid, gid)
@@ -493,7 +565,7 @@ def xauth_add(filename: str, display_name: str, xauth_data: str, uid: int, gid:
         log.estr(e)
 
 
-def check_xvfb_process(xvfb=None, cmd: str = "Xvfb", timeout: int = 0, command=None) -> bool:
+def check_xvfb_process(xvfb=None, cmd: str = "Xvfb", timeout: int = 0, command=()) -> bool:
     if xvfb is None:
         # we don't have a process to check
         return True
@@ -504,9 +576,9 @@ def check_xvfb_process(xvfb=None, cmd: str = "Xvfb", timeout: int = 0, command=N
     log.error("")
     log.error("%s command has terminated! xpra cannot continue", cmd)
     log.error(" if the display is already running, try a different one,")
-    log.error(" or use the --use-display flag")
+    log.error(" if the `xvfb` command is invalid, try a different one")
     if command:
-        log.error(" full command: %s", command)
+        log.error(" full command: %r", shlex.join(command))
     log.error("")
     return False
 
