@@ -1,28 +1,32 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 from typing import Any
-from collections.abc import Iterable
+from collections.abc import Iterable, Sequence
 
-# ensure that we use gtk as display source:
-from xpra.x11.gtk.display_source import init_gdk_display_source
-from xpra.keyboard.mask import DEFAULT_MODIFIER_MEANINGS
+from xpra.common import noop
+from xpra.keyboard.mask import DEFAULT_MODIFIER_MEANINGS, MODIFIER_MAP
 from xpra.util.objects import typedict
-from xpra.util.str_fn import std, csv, bytestostr
+from xpra.util.str_fn import std, csv, bytestostr, Ellipsizer
 from xpra.util.env import envbool
-from xpra.gtk.error import xsync, xlog
+from xpra.x11.error import xsync, xlog
 from xpra.x11.bindings.keyboard import X11KeyboardBindings
 from xpra.log import Logger
 
-init_gdk_display_source()
-X11Keyboard = X11KeyboardBindings()
-
 log = Logger("x11", "keyboard")
+verboselog = Logger("x11", "keyboard", "verbose")
 
 XKB = envbool("XPRA_XKB", True)
+
+DEFAULT_RULES = os.environ.get("XKB_DEFAULT_RULES", "evdev")
+DEFAULT_MODEL = os.environ.get("XKB_DEFAULT_MODEL", "pc105")
+DEFAULT_LAYOUT = os.environ.get("XKB_DEFAULT_LAYOUT", "us")
+DEFAULT_VARIANT = os.environ.get("XKB_DEFAULT_VARIANT", "")
+DEFAULT_OPTIONS = os.environ.get("XKB_DEFAULT_OPTIONS", "")
+
 DEBUG_KEYSYMS = [x for x in os.environ.get("XPRA_DEBUG_KEYSYMS", "").split(",") if len(x) > 0]
 
 # keys we choose not to map if the free space in the keymap is too limited
@@ -51,12 +55,24 @@ OPTIONAL_KEYS = [
 
 
 def clean_keyboard_state() -> None:
+    X11Keyboard = X11KeyboardBindings()
     with xlog:
         X11Keyboard.ungrab_all_keys()
     with xlog:
         X11Keyboard.set_layout_group(0)
+    keycodes = {}
     with xlog:
-        X11Keyboard.unpress_all_keys()
+        keycodes = X11Keyboard.get_keycodes_down()
+    if keycodes:
+        with xlog:
+            try:
+                from xpra.x11.bindings.test import XTestBindings
+            except ImportError:
+                pass
+            else:
+                XTest = XTestBindings()
+                XTest.unpress_keys(tuple(keycodes.keys()))
+
 
 ################################################################################
 # keyboard layouts
@@ -75,6 +91,9 @@ def do_set_keymap(layout: str, variant: str, options, query_struct) -> None:
     # First we try to use data from setxkbmap -query,
     # preferably as structured data:
     query_struct = typedict(query_struct)
+    rules = DEFAULT_RULES
+    model = DEFAULT_MODEL
+    layout = layout or DEFAULT_LAYOUT
     if query_struct:
         log("do_set_keymap using xkbmap_query struct=%s", query_struct)
         # The query_struct data will look something like this:
@@ -85,28 +104,33 @@ def do_set_keymap(layout: str, variant: str, options, query_struct) -> None:
         #    b"options"     : b"grp:shift_caps_toggle",
         #    }
         # parse the data into a dict:
-        rules = query_struct.strget("rules")
-        model = query_struct.strget("model")
-        layout = query_struct.strget("layout")
-        variant = query_struct.strget("variant")
-        options = query_struct.strget("options")
+        rules = query_struct.strget("rules", DEFAULT_RULES)
+        model = query_struct.strget("model", DEFAULT_MODEL)
+        layout = query_struct.strget("layout", DEFAULT_LAYOUT)
+        variant = query_struct.strget("variant", DEFAULT_VARIANT)
+        options = query_struct.strget("options", DEFAULT_OPTIONS)
         if layout:
             log.info("setting keymap: %s",
-                     csv(f"{std(k)}={std(v)}" for k, v in query_struct.items()
-                         if k in ("rules", "model", "layout", "variant", "options") and v))
+                     csv(f"{k}={std(v)}" for k, v in {
+                         "rules": rules,
+                         "model": model,
+                         "layout": layout,
+                         "variant": variant,
+                         "options": options,
+                     }.items() if v))
             if safe_setxkbmap(rules, model, layout, variant, options):
                 return
         else:
             if safe_setxkbmap(rules, model, "", "", ""):
                 return
     # fallback for non X11 clients:
-    layout = layout or "us"
-    log.info("setting keyboard layout to '%s'", std(layout))
-    safe_setxkbmap("evdev", "pc105", layout, variant, options)
+    log.info("setting keyboard layout to %r", std(layout))
+    safe_setxkbmap(rules, model, layout, variant, options)
 
 
 def safe_setxkbmap(rules: str, model: str, layout: str, variant: str, options: str):
     # (we execute the options separately in case that fails..)
+    X11Keyboard = X11KeyboardBindings()
     try:
         if X11Keyboard.setxkbmap(rules, model, layout, variant, options):
             return True
@@ -131,6 +155,7 @@ def safe_setxkbmap(rules: str, model: str, layout: str, variant: str, options: s
 
 def apply_xmodmap(instructions: list[tuple]) -> list[tuple]:
     try:
+        X11Keyboard = X11KeyboardBindings()
         with xsync:
             unset = X11Keyboard.set_xmodmap(instructions)
     except Exception as e:
@@ -143,24 +168,34 @@ def apply_xmodmap(instructions: list[tuple]) -> list[tuple]:
     return unset
 
 
-def get_keycode_mappings() -> dict[str, list[str]]:
+def get_keycode_mappings() -> dict[int, list[str]]:
+    X11Keyboard = X11KeyboardBindings()
     if XKB and X11Keyboard.hasXkb():
         return X11Keyboard.get_xkb_keycode_mappings()
     return X11Keyboard.get_keycode_mappings()
 
 
-def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes) -> dict:
-    """
-        Translate the given keycodes into the existing keymap
-    """
-    # get the list of keycodes (either from x11 keycodes or gtk keycodes):
+def get_keyval_mappings() -> dict[int, dict[int, Sequence[int]]]:
+    X11Keyboard = X11KeyboardBindings()
+    if XKB and X11Keyboard.hasXkb():
+        return X11Keyboard.get_xkb_keysym_mappings()
+    return {}
+
+
+def set_keycode_translation(xkbmap_x11_keycodes: dict, xkbmap_keycodes: Iterable) -> dict[str | tuple[int, str], int]:
+    x11_keycodes = get_keycode_mappings()
+    keycodes: dict[int, set]
     if xkbmap_x11_keycodes:
         dump_dict(xkbmap_x11_keycodes)
         keycodes = indexed_mappings(xkbmap_x11_keycodes)
     else:
         keycodes = gtk_keycodes_to_mappings(xkbmap_keycodes)
-    log("set_keycode_translation(%s, %s)", xkbmap_x11_keycodes, xkbmap_keycodes)
-    log(" keycodes=%s", keycodes)
+    log("set_keycode_translation(%s, %s)", Ellipsizer(xkbmap_x11_keycodes), Ellipsizer(xkbmap_keycodes))
+    log(" keycodes=%s", Ellipsizer(keycodes))
+    log(" x11_keycodes=%s", Ellipsizer(x11_keycodes))
+    verboselog("set_keycode_translation(%s, %s)", xkbmap_x11_keycodes, xkbmap_keycodes)
+    verboselog(" keycodes=%s", keycodes)
+    verboselog(" x11_keycodes=%s", x11_keycodes)
     """
     Example data:
     ```
@@ -168,13 +203,7 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes) -> dict:
         9: set([('', 1), ('Escape', 4), ('', 3), ('Escape', 0), ('Escape', 2)]),
         10: set([('onesuperior', 4), ('onesuperior', 8), ('exclam', 1), ('1', 6),
                  ('exclam', 3), ('1', 2), ('exclamdown', 9), ('exclamdown', 5), ('1', 0), ('exclam', 7)]),
-    ```
-    """
-    x11_keycodes = get_keycode_mappings()
-    log(" x11_keycodes=%s", x11_keycodes)
-    """
-    Example:
-    ```
+
     x11_keycodes = {
        8: ['Mode_switch', '', 'Mode_switch', '', 'Mode_switch'],
        9: ['Escape', '', 'Escape', '', 'Escape'],
@@ -182,25 +211,54 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes) -> dict:
     ```
     create faster lookup table:
     """
-    x11_keycodes_for_keysym = {}
+    trans = do_set_keycode_translation(keycodes)
+
+    if not xkbmap_x11_keycodes:
+        # now add all the keycodes we may not have mapped yet
+        # (present in the `x11_keycodes` mappings but not the `trans`lation table)
+        for keycode, keysyms in x11_keycodes.items():
+            for i, keysym in enumerate(keysyms):
+                if keysym in DEBUG_KEYSYMS:
+                    log.info("x11 keycode %s: %s", keycode, keysym)
+                # record under `keysym` and also `(keysym, index)`:
+                for trans_key in (keysym, (keysym, i)):
+                    if trans_key not in trans:
+                        trans[trans_key] = keycode
+    log("set_keycode_translation(..)=%s", Ellipsizer(trans))
+    verboselog("set_keycode_translation(..)=%s", trans)
+    return trans
+
+
+def do_set_keycode_translation(keycodes: dict[int, set]) -> dict[str | tuple[int, str], int]:
+    """
+        Translate the given keycodes into a keymap,
+        and try to preserve the existing keymap
+    """
+    x11_keycodes = get_keycode_mappings()
+    # create faster keysym lookup table:
+    x11_keycodes_for_keysym: dict[str, set[int]] = {}
     for keycode, keysyms in x11_keycodes.items():
         for keysym in keysyms:
             x11_keycodes_for_keysym.setdefault(keysym, set()).add(keycode)
 
-    def find_keycode(kc: int, keysym, i: int) -> tuple | None:
+    def find_keycode(kc: int, keysym, i: int) -> tuple:
         keycodes = tuple(x11_keycodes_for_keysym.get(keysym, set()))
-        if keysym in DEBUG_KEYSYMS:
+        if not keycodes:
+            return ()
+
+        rlog = noop
+        debug_keysym = keysym in DEBUG_KEYSYMS
+        if debug_keysym:
             log.info("set_keycode_translation: find_keycode%s x11 keycodes=%s", (kc, keysym, i), keycodes)
 
-        def rlog(keycode, msg) -> None:
-            if keysym in DEBUG_KEYSYMS:
+            def ilog(keycode, msg) -> None:
                 log.info("set_keycode_translation: find_keycode%s=%s (%s)", (kc, keysym, i), keycode, msg)
-        if not keycodes:
-            return None
+            rlog = ilog
+
         # no other option, use it:
         for keycode in keycodes:
             defs = x11_keycodes.get(keycode)
-            if keysym in DEBUG_KEYSYMS:
+            if debug_keysym:
                 log.info("server x11 keycode %i: %s", keycode, defs)
             if not defs:
                 raise RuntimeError(f"bug: keycode {keycode} not found in {x11_keycodes}")
@@ -214,38 +272,47 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes) -> dict:
         keycode = keycodes[0]
         rlog(keycode, "using first match")
         return keycode, False
+
     # generate the translation map:
-    trans = {}
+    trans: dict[str | tuple[int, str] | tuple[str, int], int] = {}
     for keycode, defs in keycodes.items():
         if bool(set(DEBUG_KEYSYMS) & set(bytestostr(d[0]) for d in defs)):
             log.info("client keycode=%i, defs=%s", keycode, defs)
         for bkeysym, i in tuple(defs):             # ie: (b'1', 0) or (b'A', 1), etc
             keysym = bytestostr(bkeysym)
             m = find_keycode(keycode, keysym, i)
-            if m:
-                x11_keycode, index_matched = m
-                trans[(keycode, keysym)] = x11_keycode
-                trans[keysym] = x11_keycode
-                if index_matched:
-                    trans[(keysym, i)] = x11_keycode
-    if not xkbmap_x11_keycodes:
-        # now add all the keycodes we may not have mapped yet
-        # (present in x11_keycodes but not keycodes)
-        for keycode, keysyms in x11_keycodes.items():
-            for i, keysym in enumerate(keysyms):
-                if keysym not in trans:
-                    if keysym in DEBUG_KEYSYMS:
-                        log.info("x11 keycode %s: %s", keycode, keysym)
-                    trans[keysym] = keycode
-                key = (keysym, i)
-                if key not in trans:
-                    if keysym in DEBUG_KEYSYMS:
-                        log.info("x11 keycode %s: %s", keycode, key)
-                    trans[key] = keycode
-    log("set_keycode_translation(..)=%s", trans)
+            if not m:
+                continue
+            x11_keycode, index_matched = m
+            trans[(keycode, keysym)] = x11_keycode
+            trans[keysym] = x11_keycode
+            if index_matched:
+                trans[(keysym, i)] = x11_keycode
     return trans
 
 
+def get_keysym_to_modifier_map(modifiers: dict[str, Iterable]) -> dict[str, str]:
+    keysym_to_modifier: dict[str, str] = {}
+    for modifier, keysyms in modifiers.items():
+        for keysym in keysyms:
+            existing_mod = keysym_to_modifier.get(keysym)
+            if existing_mod and existing_mod != modifier:
+                log.error("ERROR: keysym %s is mapped to both %s and %s !", keysym, modifier, existing_mod)
+            else:
+                keysym_to_modifier[keysym] = modifier
+                if keysym in DEBUG_KEYSYMS:
+                    log.info("set_all_keycodes() keysym_to_modifier[%s]=%s", keysym, modifier)
+    log("keysym_to_modifier=%s", keysym_to_modifier)
+    return keysym_to_modifier
+
+
+def estr(entries) -> str:
+    try:
+        return csv(tuple(set(x[0] for x in entries)))
+    except Exception:
+        return csv(tuple(entries))
+
+
 def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keycodes, modifiers: dict[str, Iterable]):
     """
         Clients that have access to raw x11 keycodes should provide
@@ -263,19 +330,10 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     """
     log("set_all_keycodes(%s.., %s.., %s.., %s)",
         str(xkbmap_x11_keycodes)[:60], str(xkbmap_keycodes)[:60], str(preserve_server_keycodes)[:60], modifiers)
+    X11Keyboard = X11KeyboardBindings()
 
     # so we can validate entries:
-    keysym_to_modifier = {}
-    for modifier, keysyms in modifiers.items():
-        for keysym in keysyms:
-            existing_mod = keysym_to_modifier.get(keysym)
-            if existing_mod and existing_mod != modifier:
-                log.error("ERROR: keysym %s is mapped to both %s and %s !", keysym, modifier, existing_mod)
-            else:
-                keysym_to_modifier[keysym] = modifier
-                if keysym in DEBUG_KEYSYMS:
-                    log.info("set_all_keycodes() keysym_to_modifier[%s]=%s", keysym, modifier)
-    log("keysym_to_modifier=%s", keysym_to_modifier)
+    keysym_to_modifier = get_keysym_to_modifier_map(modifiers)
 
     def modifiers_for(entries) -> set[str]:
         """ entries can only point to a single modifier - verify """
@@ -287,18 +345,13 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
                 modifiers.add(modifier)
             if keysym in DEBUG_KEYSYMS:
                 log_fn = log.info
+                break
         log_fn("modifiers_for(%s)=%s", entries, modifiers)
         return modifiers
 
     def filter_mappings(mappings, drop_extra_keys=False) -> dict[int, set]:
         filtered = {}
         invalid_keysyms = set()
-
-        def estr(entries) -> str:
-            try:
-                return csv(tuple(set(x[0] for x in entries)))
-            except Exception:
-                return csv(tuple(entries))
         for keycode, entries in mappings.items():
             mods = modifiers_for(entries)
             if len(mods) > 1:
@@ -357,8 +410,8 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     for try_harder in (False, True):
         filtered_preserve_keycode_entries = filter_mappings(indexed_mappings(preserve_keycode_entries), try_harder)
         log("filtered_preserve_keycode_entries=%s", filtered_preserve_keycode_entries)
-        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes,
-                                                                   filtered_preserve_keycode_entries,
+        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax,
+                                                                   keycodes, filtered_preserve_keycode_entries,
                                                                    keysym_to_modifier,
                                                                    try_harder)
         if not missing_keycodes:
@@ -369,7 +422,7 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     return trans
 
 
-def dump_dict(d) -> None:
+def dump_dict(d: dict) -> None:
     for k, v in d.items():
         log("%s\t\t=\t%s", k, v)
 
@@ -426,7 +479,9 @@ def x11_keycodes_to_list(x11_mappings: dict[int, Iterable]) -> list[tuple[str, i
     return entries
 
 
-def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder=False):
+def translate_keycodes(kcmin: int, kcmax: int, keycodes: dict[int, set],
+                       preserve_keycode_entries: dict[int, set], keysym_to_modifier: dict[str, str],
+                       try_harder=False) -> tuple[dict, dict, Sequence]:
     """
         The keycodes given may not match the range that the server supports,
         or some of those keycodes may not be usable (only one modifier can
@@ -442,9 +497,9 @@ def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entrie
     log("translate_keycodes(%s, %s, %s, %s, %s, %s)",
         kcmin, kcmax, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder)
     # list of free keycodes we can use:
-    free_keycodes = [i for i in range(kcmin, kcmax+1) if i not in preserve_keycode_entries]
-    keycode_trans = {}              # translation map from client keycode to our server keycode
-    server_keycodes = {}            # the new keycode definitions
+    free_keycodes: list[int] = [i for i in range(kcmin, kcmax+1) if i not in preserve_keycode_entries]
+    keycode_trans: dict[str | tuple[int, str], int] = {}              # translation map from client keycode to our server keycode
+    server_keycodes: dict[int, Any] = {}            # the new keycode definitions
     missing_keycodes = []           # the groups of entries we failed to map due to lack of free keycodes
 
     # to do faster lookups:
@@ -455,7 +510,7 @@ def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entrie
     for k in DEBUG_KEYSYMS:
         log.info("preserve_keysyms_map[%s]=%s", k, preserve_keysyms_map.get(k))
 
-    def do_assign(keycode, server_keycode, entries, override_server_keycode=False) -> int:
+    def do_assign(keycode: int, server_keycode: int, entries, override_server_keycode=False) -> int:
         """ may change the keycode if needed
             in which case we update the entries and populate 'keycode_trans'
         """
@@ -463,6 +518,7 @@ def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entrie
         for name, _ in entries:
             if name in DEBUG_KEYSYMS:
                 log_fn = log.info
+                break
         if (server_keycode in server_keycodes) and not override_server_keycode:
             log_fn("assign: server keycode %s already in use: %s", server_keycode, server_keycodes.get(server_keycode))
             server_keycode = -1
@@ -497,7 +553,7 @@ def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entrie
             server_keycodes[server_keycode] = entries
         return server_keycode
 
-    def assign(client_keycode, entries) -> int:
+    def assign(client_keycode: int, entries) -> int:
         if not entries:
             return 0
         # all the keysyms for this keycode:
@@ -508,7 +564,7 @@ def translate_keycodes(kcmin: int, kcmax: int, keycodes, preserve_keycode_entrie
             log("skipped invalid keysym: %s / %s", client_keycode, entries)
             return 0
         log_fn = log.debug
-        if [k for k in keysyms if k in DEBUG_KEYSYMS]:
+        if any(bool(k) for k in keysyms if k in DEBUG_KEYSYMS):
             log_fn = log.info
         log_fn("assign(%s, %s)", client_keycode, entries)
 
@@ -624,6 +680,7 @@ def keymap_to_xmodmap(trans_keycodes: dict[int, Any]) -> list[tuple]:
         for name, index in sentries:
             assert 0 <= index < keysyms_per_keycode
             try:
+                X11Keyboard = X11KeyboardBindings()
                 keysym = X11Keyboard.parse_keysym(name)
             except Exception:
                 keysym = None
@@ -657,7 +714,7 @@ def keymap_to_xmodmap(trans_keycodes: dict[int, Any]) -> list[tuple]:
         while keysyms and keysyms[0] is None:
             keysyms = keysyms[1:]
         log_fn = log.debug
-        if [k for k in keysyms if k in DEBUG_KEYSYMS]:
+        if any(bool(k) for k in keysyms if k in DEBUG_KEYSYMS):
             log_fn = log.info
         log_fn("%s: %s -> %s", server_keycode, names, keysyms)
         instructions.append(("keycode", server_keycode, keysyms))
@@ -681,6 +738,7 @@ def set_modifiers(modifiers: dict[str, Iterable[str]]) -> None:
         modifiers is a dict: {modifier : [keynames]}
         Note: the same keysym cannot appear in more than one modifier
     """
+    X11Keyboard = X11KeyboardBindings()
     instructions: list[tuple[str, str, Iterable[str]]] = []
     for modifier, keynames in modifiers.items():
         mod = X11Keyboard.parse_modifier(modifier)
@@ -767,6 +825,7 @@ def get_modifiers_from_keycodes(xkbmap_keycodes: Iterable, add_default_modifiers
 
 
 def map_missing_modifiers(keynames_for_mod: dict[str, Iterable]):
+    X11Keyboard = X11KeyboardBindings()
     x11_keycodes = X11Keyboard.get_keycode_mappings()
     min_keycode, max_keycode = X11Keyboard.get_minmax_keycodes()
     free_keycodes = [x for x in range(min_keycode, max_keycode) if x not in x11_keycodes]
@@ -793,3 +852,38 @@ def map_missing_modifiers(keynames_for_mod: dict[str, Iterable]):
     if xmodmap_changes:
         log("xmodmap_changes=%s", xmodmap_changes)
         X11Keyboard.set_xmodmap(xmodmap_changes)
+
+
+def grok_modifier_map(meanings: dict) -> dict[str, int]:
+    """
+    Return a dict mapping modifier names to corresponding X modifier bitmasks.
+    """
+    # is this still correct for GTK3?
+    modifier_map = MODIFIER_MAP.copy()
+    modifier_map |= {
+        "scroll": 0,
+        "num": 0,
+        "meta": 0,
+        "super": 0,
+        "hyper": 0,
+        "alt": 0,
+    }
+    if not meanings:
+        meanings = DEFAULT_MODIFIER_MEANINGS
+
+    from xpra.x11.xkbhelper import get_keycode_mappings
+    with xsync:
+        mappings = get_keycode_mappings()
+        X11Keyboard = X11KeyboardBindings()
+        max_keypermod, keycodes = X11Keyboard.get_modifier_map()
+    assert len(keycodes) == 8 * max_keypermod
+    for i in range(8):
+        for j in range(max_keypermod):
+            keycode = keycodes[i * max_keypermod + j]
+            if keycode:
+                keysyms = mappings.get(keycode, ())
+                for keysym in keysyms:
+                    modifier = meanings.get(keysym, "")
+                    if modifier:
+                        modifier_map[modifier] |= (1 << i)
+    return modifier_map
